#!/usr/local/bin/perl

# uidcgi.pl by Sam Nelson, February 1, 1996, v1.1
# Program written for SmallWorld Software
# REVISED February 20, 1996 by Sam Nelson, to v1.1
# -- file locking installed
#
# A library of routines for working with uid's.
#
# Interface should be very similar to the cookiefiles.
#
# WARNING:  CALLS TO &writeuidfile BY ANYBODY EXCEPT
# &setuidvalue CAN DAMAGE THE FILE LOCKING SYSTEM, WITH 
# POTENTIAL LOSS OF USER DATA.
#

require './errorcgi.pl';

$UIDFILENAME='muppet.uid';
$UIDDIRECTORY='./';
$NOUID="anonymous";
$DEBUG=1;

#************* SUB REGISTERED *****************************
#*Returns the $uid if the user is registered, and false 
#*otherwise.
#************* SUB REGISTERED *****************************

sub registered{
    &setuid;
    if (($uid) && ($uid ne $NOUID)){return $uid;}
    return 0;
}

#************* SUB SETUID *********************************
#*Similar to &setcookie, setuid _always_ sets the global
#*variable $uid.  If possible, it gets this value from
#*the cookiefile; otherwise it initializes the variable to 
#*$NOUID
#************* SUB SETUID *********************************

sub setuid{
    if (!defined($uid) || ($uid eq "")){
	$uid=$NOUID;
    }
    return $uid;
}

#************* SUB SETUIDVALUE ***************************
#*Takes two arguments, a name and a value.  Sets the pair
#*(name,value) in the uidfile associated with the current
#*global variable $uid.  On failure, calls
#*&error($BADUIDFILE).
#************* SUB SETUIDVALUE ***************************

sub setuidvalue {
    local ($name,$value,%data,%alldata);
    ($name,$value)=@_;

    # 
    # open the file
    #
    open(UIDFILE,"+<" . &uidfilename);
    &lock(UIDFILE);
    select(UIDFILE); $|=1; select(STDOUT);
    
    #
    # Make the change
    #
    %alldata=&inputuidfile(UIDFILE);
    %data=split("\t",$alldata{$uid});
    $data{$name}=$value;
    $alldata{$uid}=join("\t",%data);

    seek(UIDFILE, 0, 0);

    &printuidfile(UIDFILE,%alldata);

    truncate(UIDFILE,tell(UIDFILE));

    # 
    # close the file
    #
    &unlock(UIDFILE);
    close(UIDFILE);

    return $value;
}

#************* SUB MUSTGETUIDVALUE ***********************
#*Takes one argument, a name.  Returns the value from the 
#*pair (name,value) in the uidfile associated with the
#*current global variable $uid.  On failure calls
#*&error($BADUIDVALUE).  Values cannot be NULL.
#************* SUB MUSTGETUIDVALUE ***********************

sub mustgetuidvalue {
    local ($result);
    $result=&getuidvalue(@_);
    if (!defined($result) || ($result eq "")) {
	&error($BADuidFILE);
    }
    return $result;
}

#************* SUB GETUIDVALUE ***************************
#*Takes one argument, a name.  Returns the value from the 
#*pair (name,value) in the uid record associated with the
#*current global variable $uid.  On failure returns "".
#************* SUB GETUIDVALUE ***************************

sub getuidvalue {
    if($uid eq $NOUID){return "";}
    local ($name,%alldata,%data);
    ($name)=@_;
    %alldata=&readuidfile;
    if(!defined($alldata{$uid}) || ($alldata{$uid} eq "")){return "";}
    %data=split("\t",$alldata{$uid});
    return $data{$name};
}

#************* SUB READUIDFILE ***************************
#*Takes no arguments.  Returns an associative array of the
#*form result{uid}=data, where data is a tab-separated
#*list of (name,value) pairs.
#************* SUB READUIDFILE ***************************

sub readuidfile {
    local(%result);
    open(UIDFILE,&uidfilename);
    &lock(UIDFILE);
    select(UIDFILE); $|=1; select(STDOUT);
    while(<UIDFILE>){
	/^([^\t]*)\t(.*)\n$/;
	$result{$1}=$2;
    }
    &unlock(UIDFILE);
    close(UIDFILE);
    return(%result);
}
    
#************* SUB WRITEUIDFILE **************************
#*Takes an associative array.  Writes the array to the 
#*uid file record, overwriting any existing data.  On 
#*failure to open the uidfile, exit on $BADUIDFILE.
#*WARNING:  This subroutine has no file locking, and
#*could destroy important user data.  Writeuidfile is
#*outdated now, and probably shouldn't be used at all.
#************* SUB WRITEUIDFILE **************************

sub writeuidfile {
    local (%data);
    (%data)=@_;
    open(UIDFILE,'>'.&uidfilename) || &error($BADUIDFILE);
    &lock(UIDFILE);
    select(UIDFILE); $|=1; select(STDOUT);
    foreach $key (keys(%data)){
	print UIDFILE $key,"\t",$data{$key},"\n";
    }
    &unlock(UIDFILE);
    close(UIDFILE);
    if ($DEBUG){chmod(0777,&uidfilename);}
}

#************* SUB INPUTUIDFILE ***************************
#*Like readuidfile, except that it assumes that the file
#*is already opened, and the filehandle will be passed in.
#************* SUB INPUTUIDFILE ***************************

sub inputuidfile {
    local(%result);
    ($UIDFILE)=@_;
    while(<$UIDFILE>){
	/^([^\t]*)\t(.*)\n$/;
	$result{$1}=$2;
    }
    return(%result);
}
    
#************* SUB PRINTUIDFILE **************************
#*Like writeuidfile, but assumes that the file has already
#*been opened, and takes it passed in as its first parameter.
#************* SUB PRINTUIDFILE **************************

sub printuidfile {
    local (%data);
    ($UIDFILE,%data)=@_;
    foreach $key (keys(%data)){
	print $UIDFILE $key,"\t",$data{$key},"\n";
    }
    if ($DEBUG){chmod(0777,&uidfilename);}
}

#************* SUB UIDFILENAME ***************************
#*Takes no arguments.  Returns the name of the uidfile.
#************* SUB UIDFILENAME ***************************

sub uidfilename {
    return $UIDDIRECTORY.$UIDFILENAME;
}


#************* SUB LOCK ***************************
#*lock and unlock (from Seth) lock files
#************* SUB LOCK ***************************

sub lock
{
    $LOCK_SH = 1;
    $LOCK_EX = 2;
    $LOCK_NB = 4;
    $LOCK_UN = 8;

    local ( $file ) = @_;
    flock ( $file, $LOCK_EX );
}

sub unlock
{
    $LOCK_SH = 1;
    $LOCK_EX = 2;
    $LOCK_NB = 4;
    $LOCK_UN = 8;

    local ( $file ) = @_;
    flock ( $file, $LOCK_UN );
}
